"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
var _exportNames = {};
exports.default = void 0;
var _reactNative = require("react-native");
var _reactNativeTcpSocket = _interopRequireDefault(require("react-native-tcp-socket"));
var _reactNativeBluetoothClassic = _interopRequireDefault(require("react-native-bluetooth-classic"));
var _UsbConnection = _interopRequireDefault(require("./UsbConnection.js"));
var _NativeTscCommon = _interopRequireDefault(require("./NativeTscCommon.js"));
var _constants = require("./constants.js");
var _types = require("./types.js");
Object.keys(_types).forEach(function (key) {
  if (key === "default" || key === "__esModule") return;
  if (Object.prototype.hasOwnProperty.call(_exportNames, key)) return;
  if (key in exports && exports[key] === _types[key]) return;
  Object.defineProperty(exports, key, {
    enumerable: true,
    get: function () {
      return _types[key];
    }
  });
});
var _utils = require("./utils.js");
var _buffer = require("buffer");
function _interopRequireDefault(e) { return e && e.__esModule ? e : { default: e }; }
class Printer {
  constructor(type, connection) {
    this._type = type;
    this._connection = connection;
  }
  static async discover(types, timeout = 5000) {
    const devices = [];
    for (const type of types) {
      switch (type) {
        case _types.ConnectionType.USB:
          devices.push(...(await _UsbConnection.default.discover(timeout)));
          break;
        case _types.ConnectionType.BLUETOOTH:
          if (_reactNative.Platform.OS === 'android') {
            setTimeout(() => {
              _reactNativeBluetoothClassic.default.cancelDiscovery();
            }, timeout);
          } else {
            console.warn('Bluetooth discovery is not supported on iOS. Only bonded devices will be listed.');
          }
          const btDevices = _reactNative.Platform.OS === 'android' ? await _reactNativeBluetoothClassic.default.startDiscovery() : await _reactNativeBluetoothClassic.default.getBondedDevices();
          devices.push(...btDevices.filter(device => device.type === 'CLASSIC').map(device => ({
            name: device.name,
            address: device.address,
            target: device.address,
            type: _types.ConnectionType.BLUETOOTH
          })));
          break;
        default:
          throw new Error('Unsupported connection type');
      }
    }
    return devices;
  }
  static async connect(type, target) {
    switch (type) {
      case _types.ConnectionType.USB:
        return new Printer(type, await _UsbConnection.default.connect(target));
      case _types.ConnectionType.NET:
        {
          return new Promise((resolve, reject) => {
            const [host, port] = target.split(':');
            const socket = _reactNativeTcpSocket.default.connect({
              host,
              port: Number(port)
            }, () => {
              resolve(new Printer(type, socket));
            });
            socket.on('error', reject);
          });
        }
      case _types.ConnectionType.BLUETOOTH:
        {
          const available = await _reactNativeBluetoothClassic.default.isBluetoothAvailable();
          if (!available) throw new Error('Bluetooth is not available');
          const enabled = await _reactNativeBluetoothClassic.default.isBluetoothEnabled();
          if (!enabled) throw new Error('Bluetooth is not enabled');
          const pairedDevices = await _reactNativeBluetoothClassic.default.getBondedDevices();
          let device = pairedDevices.find(dev => dev.address === target);
          if (!device && _reactNative.Platform.OS === 'ios') throw new Error('The device is not paired');
          if (!device) device = await _reactNativeBluetoothClassic.default.pairDevice(target);
          if (!(await device.isConnected())) await device.connect({
            connectionType: 'rfcomm',
            delimiter: '\r\n'
          });
          return new Printer(type, device);
        }
      default:
        throw new Error('Unsupported connection type');
    }
  }
  async disconnect() {
    switch (this._type) {
      case _types.ConnectionType.USB:
        await this._connection.disconnect();
        break;
      case _types.ConnectionType.NET:
        await this._connection.destroy();
        break;
      case _types.ConnectionType.BLUETOOTH:
        await this._connection.disconnect();
        break;
    }
  }
  async receive(timeout = 1000) {
    switch (this._type) {
      case _types.ConnectionType.USB:
        return await this._connection.read();
      case _types.ConnectionType.BLUETOOTH:
        return await this._connection.read();
      case _types.ConnectionType.NET:
        return new Promise((resolve, reject) => {
          const timeoutId = setTimeout(() => reject(new Error('Timeout')), timeout);
          this._connection.once('data', data => {
            clearTimeout(timeoutId);
            resolve(data);
          });
        });
      default:
        throw new Error('Unsupported connection type');
    }
  }
  async sendCommand(command) {
    if (!_buffer.Buffer.isBuffer(command)) {
      command = _buffer.Buffer.from(command);
    }
    switch (this._type) {
      case _types.ConnectionType.USB:
        await this._connection.send(command);
        break;
      case _types.ConnectionType.NET:
        return new Promise((resolve, reject) => {
          this._connection.write(command, 'binary', err => {
            if (err) reject(err);else resolve();
          });
        });
      case _types.ConnectionType.BLUETOOTH:
        await this._connection.write(command, 'binary');
        break;
      default:
    }
  }
  async clearBuffer() {
    return this.sendCommand((0, _utils.buildCommand)('CLS'));
  }
  async setSize(width, height) {
    if (!Number.isInteger(width) || !Number.isInteger(height)) {
      throw new Error('Width and height must be integers');
    }
    return this.sendCommand((0, _utils.buildCommand)('SIZE', `${width} mm`, `${height} mm`));
  }
  async setSpeed(speed) {
    if (!Number.isInteger(speed)) {
      throw new Error('Speed must be an integer');
    }
    return this.sendCommand((0, _utils.buildCommand)('SPEED', speed));
  }
  async setDensity(density) {
    if (!Number.isInteger(density)) {
      throw new Error('Density must be an integer');
    }
    return this.sendCommand((0, _utils.buildCommand)('DENSITY', density));
  }
  async setSensor(sensor, distance, offset) {
    if (!Number.isInteger(distance) || !Number.isInteger(offset)) {
      throw new Error('Distance and offset must be integers');
    }
    switch (sensor) {
      case _types.SensorType.Gap:
        return this.sendCommand((0, _utils.buildCommand)('GAP', `${distance} mm`, `${offset} mm`));
      case _types.SensorType.BlackLine:
        return this.sendCommand((0, _utils.buildCommand)('BLINE', `${distance} mm`, `${offset} mm`));
      default:
        throw new Error('Unsupported sensor type');
    }
  }
  async setCodepage(codepage) {
    return this.sendCommand((0, _utils.buildCommand)('CODEPAGE', codepage));
  }
  async addBarcode(x, y, type, content, size, {
    hri = false,
    rotation = 0,
    narrow,
    wide
  } = {}) {
    wide ??= _constants.BARCODE_DEFAULT_WIDE[type];
    narrow ??= type === _types.BarcodeType.CPOST ? 3 : 1;
    return this.sendCommand((0, _utils.buildCommand)('BARCODE', x, y, type, size, hri ? 1 : 0, rotation, narrow, wide, (0, _utils.quote)(content)));
  }
  async addText(x, y, content, {
    font = _types.Font.MonotyeCG0,
    rotation = 0,
    magnification,
    alignment = _types.TextAlignment.Default
  } = {}) {
    return this.sendCommand((0, _utils.buildCommand)('TEXT', x, y, (0, _utils.quote)(font), rotation, magnification?.x ?? 1, magnification?.y ?? 1, alignment, (0, _utils.quote)(content)));
  }
  async addQRCode(x, y, content, cell = 1, {
    level = 'L',
    mode = _types.QRCodeMode.Auto,
    rotation = 0,
    model = _types.QRCodeModel.Model1,
    mask = _types.QRCodeMask.S7
  } = {}) {
    return this.sendCommand((0, _utils.buildCommand)('QRCODE', x, y, level, cell, mode, rotation, model, mask, (0, _utils.quote)(content)));
  }
  async addBar(x, y, width, height) {
    return this.sendCommand((0, _utils.buildCommand)('BAR', x, y, width, height));
  }
  async print(quantity = 1, copies = 1) {
    return this.sendCommand((0, _utils.buildCommand)('PRINT', quantity, copies));
  }
  async feed(dots = 1) {
    return this.sendCommand((0, _utils.buildCommand)('FEED', dots));
  }
  async formfeed() {
    return this.sendCommand((0, _utils.buildCommand)('FORMFEED'));
  }
  async backfeed(dots = 1) {
    return this.sendCommand((0, _utils.buildCommand)('BACKFEED', dots));
  }
  async cut() {
    return this.sendCommand((0, _utils.buildCommand)('CUT'));
  }
  async noBackfeed() {
    return this.sendCommand((0, _utils.buildCommand)('SET TEAR OFF'));
  }
  async addImage(x, y, uri, heightInDots = 0) {
    const {
      data,
      widthBytes
    } = await _NativeTscCommon.default.loadImage(uri, heightInDots);
    return this.addBitmap(x, y, widthBytes, heightInDots, new Uint8Array(data));
  }
  async addBitmap(x, y, widthBytes, heightInDots, bitmap) {
    const mode = 0;
    return this.sendCommand((0, _utils.buildCommand)('BITMAP', x, y, widthBytes, heightInDots, mode, _buffer.Buffer.from(bitmap)));
  }
  async restart() {
    await this.sendCommand(_buffer.Buffer.from([27, 33, 82]));
  }
  async getStatus() {
    await this.sendCommand(_buffer.Buffer.from([27, 33, 63]));
    const status = await this.receive();
    return _constants.STATUS_MAP[status.toString()] ?? 'Unknown';
  }
  async getPrinterName() {
    await this.sendCommand('~!T');
    return (await this.receive()).toString();
  }
  async getPrinterMemory() {
    await this.sendCommand('~!A');
    return (await this.receive()).toString();
  }
  async getPrinterMileage() {
    await this.sendCommand('~!@');
    return (await this.receive()).toString();
  }
  async getPrinterCodepage() {
    await this.sendCommand('~!I');
    return (await this.receive()).toString();
  }
}
var _default = exports.default = Printer;
//# sourceMappingURL=index.js.map