"use strict";

import { Buffer } from 'buffer';
import { EventEmitter } from 'tseep';
import { usbEventEmitter } from "./EventEmitters.js";
import UsbApi from "./NativeTscUsb.js";
import { ConnectionType } from "./types.js";
class UsbConnection extends EventEmitter {
  static async discover(timeout) {
    if (!UsbApi) throw new Error('The TSC USB is not enabled.');
    const devices = await UsbApi.scanDevices(timeout);
    return devices.map(device => ({
      ...device,
      type: ConnectionType.USB
    }));
  }
  constructor(id) {
    super();
    this._id = id;
    if (!usbEventEmitter) throw new Error('The TSC USB is not enabled.');
    this._usbDeviceAttachedListener = usbEventEmitter.addListener('connected', event => {
      if (event.id === this._id) {
        this.emit('connected');
      }
    });
    this._usbDeviceDetachedListener = usbEventEmitter.addListener('disconnected', event => {
      if (event.id === this._id) {
        this.emit('disconnected');
      }
    });
    this._usbDeviceDataListener = usbEventEmitter.addListener('data', event => {
      if (event.id === this._id) {
        this.emit('data', event.data);
      }
    });
  }
  static async connect(target) {
    if (!UsbApi) throw new Error('The TSC USB is not enabled.');
    return new UsbConnection(await UsbApi.connect(target));
  }
  async disconnect() {
    await UsbApi.disconnect(this._id);
    this._usbDeviceAttachedListener.remove();
    this._usbDeviceDetachedListener.remove();
    this._usbDeviceDataListener.remove();
  }
  async send(data) {
    await UsbApi.send(this._id, data.toString('base64'));
  }
  async read() {
    return Buffer.from(await UsbApi.read(this._id), 'base64');
  }
}
export default UsbConnection;
//# sourceMappingURL=UsbConnection.js.map