"use strict";

import { Platform } from 'react-native';
import net from 'react-native-tcp-socket';
import BluetoothClassic from 'react-native-bluetooth-classic';
import UsbConnection from "./UsbConnection.js";
import Common from "./NativeTscCommon.js";
import { BARCODE_DEFAULT_WIDE, STATUS_MAP } from "./constants.js";
import { ConnectionType, SensorType, Font, TextAlignment, BarcodeType, QRCodeMode, QRCodeModel, QRCodeMask } from "./types.js";
import { buildCommand, quote } from "./utils.js";
import { Buffer } from 'buffer';
export * from "./types.js";
class Printer {
  constructor(type, connection) {
    this._type = type;
    this._connection = connection;
  }
  static async discover(types, timeout = 5000) {
    const devices = [];
    for (const type of types) {
      switch (type) {
        case ConnectionType.USB:
          devices.push(...(await UsbConnection.discover(timeout)));
          break;
        case ConnectionType.BLUETOOTH:
          if (Platform.OS === 'android') {
            setTimeout(() => {
              BluetoothClassic.cancelDiscovery();
            }, timeout);
          } else {
            console.warn('Bluetooth discovery is not supported on iOS. Only bonded devices will be listed.');
          }
          const btDevices = Platform.OS === 'android' ? await BluetoothClassic.startDiscovery() : await BluetoothClassic.getBondedDevices();
          devices.push(...btDevices.filter(device => device.type === 'CLASSIC').map(device => ({
            name: device.name,
            address: device.address,
            target: device.address,
            type: ConnectionType.BLUETOOTH
          })));
          break;
        default:
          throw new Error('Unsupported connection type');
      }
    }
    return devices;
  }
  static async connect(type, target) {
    switch (type) {
      case ConnectionType.USB:
        return new Printer(type, await UsbConnection.connect(target));
      case ConnectionType.NET:
        {
          return new Promise((resolve, reject) => {
            const [host, port] = target.split(':');
            const socket = net.connect({
              host,
              port: Number(port)
            }, () => {
              resolve(new Printer(type, socket));
            });
            socket.on('error', reject);
          });
        }
      case ConnectionType.BLUETOOTH:
        {
          const available = await BluetoothClassic.isBluetoothAvailable();
          if (!available) throw new Error('Bluetooth is not available');
          const enabled = await BluetoothClassic.isBluetoothEnabled();
          if (!enabled) throw new Error('Bluetooth is not enabled');
          const pairedDevices = await BluetoothClassic.getBondedDevices();
          let device = pairedDevices.find(dev => dev.address === target);
          if (!device && Platform.OS === 'ios') throw new Error('The device is not paired');
          if (!device) device = await BluetoothClassic.pairDevice(target);
          if (!(await device.isConnected())) await device.connect({
            connectionType: 'rfcomm',
            delimiter: '\r\n'
          });
          return new Printer(type, device);
        }
      default:
        throw new Error('Unsupported connection type');
    }
  }
  async disconnect() {
    switch (this._type) {
      case ConnectionType.USB:
        await this._connection.disconnect();
        break;
      case ConnectionType.NET:
        await this._connection.destroy();
        break;
      case ConnectionType.BLUETOOTH:
        await this._connection.disconnect();
        break;
    }
  }
  async receive(timeout = 1000) {
    switch (this._type) {
      case ConnectionType.USB:
        return await this._connection.read();
      case ConnectionType.BLUETOOTH:
        return await this._connection.read();
      case ConnectionType.NET:
        return new Promise((resolve, reject) => {
          const timeoutId = setTimeout(() => reject(new Error('Timeout')), timeout);
          this._connection.once('data', data => {
            clearTimeout(timeoutId);
            resolve(data);
          });
        });
      default:
        throw new Error('Unsupported connection type');
    }
  }
  async sendCommand(command) {
    if (!Buffer.isBuffer(command)) {
      command = Buffer.from(command);
    }
    switch (this._type) {
      case ConnectionType.USB:
        await this._connection.send(command);
        break;
      case ConnectionType.NET:
        return new Promise((resolve, reject) => {
          this._connection.write(command, 'binary', err => {
            if (err) reject(err);else resolve();
          });
        });
      case ConnectionType.BLUETOOTH:
        await this._connection.write(command, 'binary');
        break;
      default:
    }
  }
  async clearBuffer() {
    return this.sendCommand(buildCommand('CLS'));
  }
  async setSize(width, height) {
    if (!Number.isInteger(width) || !Number.isInteger(height)) {
      throw new Error('Width and height must be integers');
    }
    return this.sendCommand(buildCommand('SIZE', `${width} mm`, `${height} mm`));
  }
  async setSpeed(speed) {
    if (!Number.isInteger(speed)) {
      throw new Error('Speed must be an integer');
    }
    return this.sendCommand(buildCommand('SPEED', speed));
  }
  async setDensity(density) {
    if (!Number.isInteger(density)) {
      throw new Error('Density must be an integer');
    }
    return this.sendCommand(buildCommand('DENSITY', density));
  }
  async setSensor(sensor, distance, offset) {
    if (!Number.isInteger(distance) || !Number.isInteger(offset)) {
      throw new Error('Distance and offset must be integers');
    }
    switch (sensor) {
      case SensorType.Gap:
        return this.sendCommand(buildCommand('GAP', `${distance} mm`, `${offset} mm`));
      case SensorType.BlackLine:
        return this.sendCommand(buildCommand('BLINE', `${distance} mm`, `${offset} mm`));
      default:
        throw new Error('Unsupported sensor type');
    }
  }
  async setCodepage(codepage) {
    return this.sendCommand(buildCommand('CODEPAGE', codepage));
  }
  async addBarcode(x, y, type, content, size, {
    hri = false,
    rotation = 0,
    narrow,
    wide
  } = {}) {
    wide ??= BARCODE_DEFAULT_WIDE[type];
    narrow ??= type === BarcodeType.CPOST ? 3 : 1;
    return this.sendCommand(buildCommand('BARCODE', x, y, type, size, hri ? 1 : 0, rotation, narrow, wide, quote(content)));
  }
  async addText(x, y, content, {
    font = Font.MonotyeCG0,
    rotation = 0,
    magnification,
    alignment = TextAlignment.Default
  } = {}) {
    return this.sendCommand(buildCommand('TEXT', x, y, quote(font), rotation, magnification?.x ?? 1, magnification?.y ?? 1, alignment, quote(content)));
  }
  async addQRCode(x, y, content, cell = 1, {
    level = 'L',
    mode = QRCodeMode.Auto,
    rotation = 0,
    model = QRCodeModel.Model1,
    mask = QRCodeMask.S7
  } = {}) {
    return this.sendCommand(buildCommand('QRCODE', x, y, level, cell, mode, rotation, model, mask, quote(content)));
  }
  async addBar(x, y, width, height) {
    return this.sendCommand(buildCommand('BAR', x, y, width, height));
  }
  async print(quantity = 1, copies = 1) {
    return this.sendCommand(buildCommand('PRINT', quantity, copies));
  }
  async feed(dots = 1) {
    return this.sendCommand(buildCommand('FEED', dots));
  }
  async formfeed() {
    return this.sendCommand(buildCommand('FORMFEED'));
  }
  async backfeed(dots = 1) {
    return this.sendCommand(buildCommand('BACKFEED', dots));
  }
  async cut() {
    return this.sendCommand(buildCommand('CUT'));
  }
  async noBackfeed() {
    return this.sendCommand(buildCommand('SET TEAR OFF'));
  }
  async addImage(x, y, uri, heightInDots = 0) {
    const {
      data,
      widthBytes
    } = await Common.loadImage(uri, heightInDots);
    return this.addBitmap(x, y, widthBytes, heightInDots, new Uint8Array(data));
  }
  async addBitmap(x, y, widthBytes, heightInDots, bitmap) {
    const mode = 0;
    return this.sendCommand(buildCommand('BITMAP', x, y, widthBytes, heightInDots, mode, Buffer.from(bitmap)));
  }
  async restart() {
    await this.sendCommand(Buffer.from([27, 33, 82]));
  }
  async getStatus() {
    await this.sendCommand(Buffer.from([27, 33, 63]));
    const status = await this.receive();
    return STATUS_MAP[status.toString()] ?? 'Unknown';
  }
  async getPrinterName() {
    await this.sendCommand('~!T');
    return (await this.receive()).toString();
  }
  async getPrinterMemory() {
    await this.sendCommand('~!A');
    return (await this.receive()).toString();
  }
  async getPrinterMileage() {
    await this.sendCommand('~!@');
    return (await this.receive()).toString();
  }
  async getPrinterCodepage() {
    await this.sendCommand('~!I');
    return (await this.receive()).toString();
  }
}
export default Printer;
//# sourceMappingURL=index.js.map